﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Grids;
using System.Diagnostics;
using System.Windows.Threading;

namespace XamGridHorizontalScrolling
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public int colCount { get; set; }
        public int rowCount { get; set; }
        public bool useGroups { get; set; }


        public List<object> Data
        {
            get { return (List<object>)GetValue(DataProperty); }
            set { SetValue(DataProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Data.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty DataProperty =
            DependencyProperty.Register("Data", typeof(List<object>), typeof(MainWindow), new UIPropertyMetadata(null));



        public MainWindow()
        {
            colCount = 40;
            rowCount = 1000;
            useGroups = true;
            InitializeComponent();
            MainGrid.DeferredScrollingSettings.AllowDeferredScrolling = DeferredScrollingType.Default;
            this.DataContext = this;
            MainGrid.ColumnLayouts.Add(GenerateColumnLayout());

            MainGrid.SelectionSettings.CellClickAction = CellSelectionAction.SelectRow;
            MainGrid.SelectionSettings.RowSelection = SelectionType.Multiple;
            MainGrid.SelectedRowsCollectionChanged += new EventHandler<SelectionCollectionChangedEventArgs<SelectedRowsCollection>>(MainGrid_SelectedRowsCollectionChanged);
            
            Data = GetData();
        }

        void MainGrid_SelectedRowsCollectionChanged(object sender, SelectionCollectionChangedEventArgs<SelectedRowsCollection> e)
        {
            Debug.WriteLine(e.NewSelectedItems.Count);
        }

        private ColumnLayout GenerateColumnLayout()
        {
            ColumnLayout columnLayout = new ColumnLayout() { Key = "XamWebGrid_Root_XamWebGrid" };
            var r = new Random();
            var cols = new List<Column>();
            for (int i = 0; i < colCount; i++)
            {
                cols.Add(new TemplateColumn()
                {
                    AllowToolTips = Infragistics.Controls.Grids.AllowToolTips.Always,
                    HeaderText = "Column "+i,
                    Key = "[Column "+i+"]",
                    Tag = new Dictionary<string,object>() { {"Metadata",new Tuple<string>("Column "+i)}}
                });
            }

            if (useGroups)
            {
                var id = 0;
                var num = 5;
                while (cols.Count > 2 * num)
                {
                    var subs = new List<Column>();
                    for (int i = 0; i < num; i++)
                    {
                        subs.Add(cols[cols.Count - 1]);
                        cols.RemoveAt(cols.Count - 1);
                    }

                    var group = new GroupColumn() { HeaderText = "Group Header" + (++id), Key = id + "" };
                    foreach (var col in subs)
                        group.Columns.Add(col);
                    cols.Add(group);

                    subs = new List<Column>();
                    for (int i = 0; i < num; i++)
                    {
                        subs.Add(cols[0]);
                        cols.RemoveAt(0);
                    }

                    group = new GroupColumn() { HeaderText = "Group Header" + (++id), Key = id + "" };
                    foreach (var col in subs)
                        group.Columns.Add(col);
                    cols.Add(group);
                }
            }

            foreach (var col in cols)
                columnLayout.Columns.Add(col);
            return columnLayout;
        }
        public List<object> GetData()
        {
            var list = new List<object>(rowCount);
            for (int i = 0; i < rowCount; i++)
            {
                var dict = new Dictionary<string, object>();
                for (int j = 0; j < colCount; j++)
                    dict["Column " + j] = i*100 + j;
                list.Add(dict);
            }
            return list;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Data = null;
            var count = MainGrid.ColumnLayouts.Count;
            for (int i = 0; i < count; i++)
                MainGrid.ColumnLayouts.RemoveAt(0);

            MainGrid.ColumnLayouts.Add(GenerateColumnLayout());
            Data = GetData();
        }
    }
}
